@@ -8,7 +8,7 @@ import userpath  # type: ignore
 
 from pipx import constants
 from pipx.constants import EXIT_CODE_OK, ExitCode
-from pipx.emojies import hazard, stars
+from pipx.emojis import hazard, stars
 from pipx.util import pipx_wrap
 
 logger = logging.getLogger(__name__)
@@ -61,7 +61,7 @@ def ensure_path(location: Path, *, force: bool) -> Tuple[bool, bool]:
     in_current_path = userpath.in_current_path(location_str)
 
     if force or (not in_current_path and not need_shell_restart):
-        userpath.append(location_str)
+        userpath.append(location_str, "pipx")
         print(
             pipx_wrap(
                 f"Success! Added {location_str} to the PATH environment variable.",
@@ -91,11 +91,11 @@ def ensure_path(location: Path, *, force: bool) -> Tuple[bool, bool]:
 
 def ensure_pipx_paths(force: bool) -> ExitCode:
     """Returns pipx exit code."""
-    bin_paths = [constants.LOCAL_BIN_DIR]
+    bin_paths = set([constants.LOCAL_BIN_DIR])
 
     pipx_user_bin_path = get_pipx_user_bin_path()
     if pipx_user_bin_path is not None:
-        bin_paths.append(pipx_user_bin_path)
+        bin_paths.add(pipx_user_bin_path)
 
     path_added = False
     need_shell_restart = False
