@@ -1,7 +1,9 @@
 import logging
 import os
+import random
 import re
 import shutil
+import string
 import subprocess
 import sys
 import textwrap
@@ -21,7 +23,7 @@ from typing import (
 
 import pipx.constants
 from pipx.animate import show_cursor
-from pipx.constants import WINDOWS
+from pipx.constants import PIPX_TRASH_DIR, WINDOWS
 
 logger = logging.getLogger(__name__)
 
@@ -34,7 +36,22 @@ class PipxError(Exception):
             super().__init__(message)
 
 
-def rmdir(path: Path) -> None:
+class RelevantSearch(NamedTuple):
+    pattern: Pattern[str]
+    category: str
+
+
+def _get_trash_file(path: Path) -> Path:
+    if not PIPX_TRASH_DIR.is_dir():
+        PIPX_TRASH_DIR.mkdir()
+    prefix = "".join(random.choices(string.ascii_lowercase, k=8))
+    return PIPX_TRASH_DIR / f"{prefix}.{path.name}"
+
+
+def rmdir(path: Path, safe_rm: bool = True) -> None:
+    if not path.is_dir():
+        return
+
     logger.info(f"removing directory {path}")
     try:
         if WINDOWS:
@@ -44,6 +61,19 @@ def rmdir(path: Path) -> None:
     except FileNotFoundError:
         pass
 
+    # move it to be deleted later if it still exists
+    if path.is_dir():
+        if safe_rm:
+            logger.warning(
+                f"Failed to delete {path}. Will move it to a temp folder to delete later."
+            )
+
+            path.rename(_get_trash_file(path))
+        else:
+            logger.warning(
+                f"Failed to delete {path}. You may need to delete it manually."
+            )
+
 
 def mkdir(path: Path) -> None:
     if path.is_dir():
@@ -52,6 +82,19 @@ def mkdir(path: Path) -> None:
     path.mkdir(parents=True, exist_ok=True)
 
 
+def safe_unlink(file: Path) -> None:
+    # Windows doesn't let us delete or overwrite files that are being run
+    # But it does let us rename/move it. To get around this issue, we can move
+    # the file to a temporary folder (to be deleted at a later time)
+
+    if not file.is_file():
+        return
+    try:
+        file.unlink()
+    except PermissionError:
+        file.rename(_get_trash_file(file))
+
+
 def get_pypackage_bin_path(binary_name: str) -> Path:
     return (
         Path("__pypackages__")
@@ -76,7 +119,6 @@ if WINDOWS:
         python_path = bin_path / "python.exe"
         return bin_path, python_path
 
-
 else:
 
     def get_venv_paths(root: Path) -> Tuple[Path, Path]:
@@ -108,6 +150,7 @@ def _fix_subprocess_env(env: Dict[str, str]) -> Dict[str, str]:
     env["PIP_DISABLE_PIP_VERSION_CHECK"] = "1"
     # Make sure that Python writes output in UTF-8
     env["PYTHONIOENCODING"] = "utf-8"
+    env["PYTHONLEGACYWINDOWSSTDIO"] = "utf-8"
     # Make sure we install package to venv, not userbase dir
     env["PIP_USER"] = "0"
     return env
@@ -120,7 +163,7 @@ def run_subprocess(
     log_cmd_str: Optional[str] = None,
     log_stdout: bool = True,
     log_stderr: bool = True,
-) -> subprocess.CompletedProcess:
+) -> "subprocess.CompletedProcess[str]":
     """Run arbitrary command as subprocess, capturing stderr and stout"""
     env = dict(os.environ)
     env = _fix_subprocess_env(env)
@@ -149,7 +192,7 @@ def run_subprocess(
 
 
 def subprocess_post_check(
-    completed_process: subprocess.CompletedProcess, raise_error: bool = True
+    completed_process: "subprocess.CompletedProcess[str]", raise_error: bool = True
 ) -> None:
     if completed_process.returncode:
         if completed_process.stdout is not None:
@@ -175,7 +218,7 @@ def dedup_ordered(input_list: List[Any]) -> List[Any]:
     return output_list
 
 
-def analyze_pip_output(pip_stdout: str, pip_stderr: str):
+def analyze_pip_output(pip_stdout: str, pip_stderr: str) -> None:
     r"""Extract useful errors from pip output of failed install
 
     Print the module that failed to build
@@ -201,7 +244,7 @@ def analyze_pip_output(pip_stdout: str, pip_stderr: str):
     """
     max_relevant_errors = 10
 
-    failed_build_stdout = []
+    failed_build_stdout: List[str] = []
     last_collecting_dep: Optional[str] = None
     # for any useful information in stdout, `pip install` must be run without
     #   the -q option
@@ -213,10 +256,6 @@ def analyze_pip_output(pip_stdout: str, pip_stderr: str):
         if collecting_match:
             last_collecting_dep = collecting_match.group(1)
 
-    class RelevantSearch(NamedTuple):
-        re: Pattern
-        type_: str
-
     # In order of most useful to least useful
     relevant_searches = [
         RelevantSearch(re.compile(r"not (?:be )?found", re.I), "not_found"),
@@ -248,8 +287,8 @@ def analyze_pip_output(pip_stdout: str, pip_stderr: str):
             failed_build_stderr.add(failed_build_match.group(1))
 
         for relevant_search in relevant_searches:
-            if relevant_search.re.search(line):
-                relevants_saved.append((line.strip(), relevant_search.type_))
+            if relevant_search.pattern.search(line):
+                relevants_saved.append((line.strip(), relevant_search.category))
                 break
 
     if failed_build_stdout:
@@ -275,7 +314,7 @@ def analyze_pip_output(pip_stdout: str, pip_stderr: str):
     if relevants_saved:
         print("\nSome possibly relevant errors from pip install:", file=sys.stderr)
 
-        print_categories = [x.type_ for x in relevant_searches]
+        print_categories = [x.category for x in relevant_searches]
         relevants_saved_filtered = relevants_saved.copy()
         while (len(print_categories) > 1) and (
             len(relevants_saved_filtered) > max_relevant_errors
@@ -290,7 +329,7 @@ def analyze_pip_output(pip_stdout: str, pip_stderr: str):
 
 
 def subprocess_post_check_handle_pip_error(
-    completed_process: subprocess.CompletedProcess,
+    completed_process: "subprocess.CompletedProcess[str]",
 ) -> None:
     if completed_process.returncode:
         logger.info(f"{' '.join(completed_process.args)!r} failed")
@@ -300,7 +339,7 @@ def subprocess_post_check_handle_pip_error(
         pip_error_file = pipx.constants.pipx_log_file.parent / (
             pipx.constants.pipx_log_file.stem + "_pip_errors.log"
         )
-        with pip_error_file.open("w") as pip_error_fh:
+        with pip_error_file.open("w", encoding="utf-8") as pip_error_fh:
             print("PIP STDOUT", file=pip_error_fh)
             print("----------", file=pip_error_fh)
             if completed_process.stdout is not None:
@@ -325,7 +364,7 @@ def exec_app(
 ) -> NoReturn:
     """Run command, do not return
 
-    POSIX: replace current processs with command using os.exec*()
+    POSIX: replace current process with command using os.exec*()
     Windows: Use subprocess and sys.exit() to run command
     """
 
@@ -363,11 +402,11 @@ def exec_app(
         os.execvpe(str(cmd[0]), [str(x) for x in cmd], env)
 
 
-def full_package_description(package: str, package_spec: str) -> str:
-    if package == package_spec:
-        return package
+def full_package_description(package_name: str, package_spec: str) -> str:
+    if package_name == package_spec:
+        return package_name
     else:
-        return f"{package} from spec {package_spec!r}"
+        return f"{package_name} from spec {package_spec!r}"
 
 
 def pipx_wrap(
